import PINRemoteImage
import UIKit

class PhotoViewerViewController: UIViewController {
    enum Object {
        case image(UIImage)
        case animatedImage(PINCachedAnimatedImage)
        case urlString(String)
    }
    
    private(set) var viewerObject: Object

    @IBOutlet weak var imageView: PINAnimatedImageView?
    @IBOutlet weak var scrollView: UIScrollView?
    @IBOutlet weak var topConstraint: NSLayoutConstraint?
    @IBOutlet weak var leftConstraint: NSLayoutConstraint?
    
    private var initialZoomScale: CGFloat = 0
    private var initialImageFullScreenFrame: CGRect = .zero
    private var panPivot: CGPoint = .zero

    init(viewerObject: Object) {
        self.viewerObject = viewerObject
        super.init(nibName: nil, bundle: nil)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.edgesForExtendedLayout = .all
        
        self.scrollView?.delegate = self
        
        switch self.viewerObject {
        case let .image(image):
            self.imageView?.image = image
        case let .animatedImage(animatedImage):
//            self.imageView?.image = animatedImage.coverImage
            self.imageView?.animatedImage = animatedImage
            self.view.setNeedsLayout()
        case let .urlString(url):
            self.loadImage(url: url)
        }
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        
        self.initialZoomScale = self.scrollView?.zoomScale ?? 1
        self.centerScrollViewContents()
    }
    
    override func viewDidLayoutSubviews() {
        super.viewDidLayoutSubviews()
        
        let photoImage: UIImage? = {
            guard case let .image(image) = self.viewerObject else {
                return self.imageView?.image// ?? self.imageView?.animatedImage?.coverImage
            }
            
            return image
        }()
        
        guard let image = photoImage, let scrollRect = self.scrollView?.frame else {
            return
        }
        
        guard image.size.width > 0 && image.size.height > 0 else {
            return
        }
        
        let scaleX = scrollRect.width / image.size.width
        let scaleY = scrollRect.height / image.size.height
        
        let minScale = scaleX > scaleY ? scaleY : scaleX
        
        self.scrollView?.minimumZoomScale = minScale
        self.scrollView?.zoomScale = minScale
        self.scrollView?.maximumZoomScale = minScale * 2
        self.initialZoomScale = minScale
        
        self.centerScrollViewContents()
    }
    
    private func loadImage(url: String) {
        self.imageView?.pin_setImage(from: URL(string: url), completion: { [weak self] (result) in
            guard let self = self else { return }
            
//            if let animatedImage = result.alternativeRepresentation as? PINCachedAnimatedImage {
//                self.imageView?.image = animatedImage.coverImage
//                self.imageView?.animatedImage = animatedImage
//            }
            
            self.view.setNeedsLayout()
        })
    }
    
    private func centerScrollViewContents() {
        let boundSize = self.scrollView?.bounds.size ?? .zero
        let contentsFrame = self.imageView?.frame ?? .zero
        
        var left: CGFloat = 0
        var top: CGFloat = 0
        
        if contentsFrame.size.width < boundSize.width {
            left = (boundSize.width - contentsFrame.size.width) / 2
        }
        
        if contentsFrame.size.height < boundSize.height {
            top = (boundSize.height - contentsFrame.size.height) / 2
        }
        
        self.topConstraint?.constant = top
        self.leftConstraint?.constant = left
        
        self.view.layoutIfNeeded()
    }
    
    override var prefersStatusBarHidden: Bool {
        return true
    }
}

// MARK: - Actions

extension PhotoViewerViewController {
    @IBAction func tappedImage(_ sender: UITapGestureRecognizer) {
        if self.scrollView?.zoomScale != self.initialZoomScale {
            UIView.animate(withDuration: 0.25) {
                self.centerScrollViewContents()
            }
            self.scrollView?.setZoomScale(self.initialZoomScale, animated: true)
            return
        }
        
        self.dismiss(animated: true, completion: nil)
    }
}

// MARK: - UIGestureRecognizerDelegate

extension PhotoViewerViewController: UIGestureRecognizerDelegate {
    func gestureRecognizer(_ gestureRecognizer: UIGestureRecognizer, shouldReceive touch: UITouch) -> Bool {
        guard gestureRecognizer is UITapGestureRecognizer else {
            return touch.view == self.imageView
        }
        
        return true
    }
}

// MARK: - UIScrollViewDelegate

extension PhotoViewerViewController: UIScrollViewDelegate {
    func viewForZooming(in scrollView: UIScrollView) -> UIView? {
        return self.imageView
    }
    
    func scrollViewDidZoom(_ scrollView: UIScrollView) {
        self.centerScrollViewContents()
    }
}
